@@ -143,6 +143,19 @@ class CursorFlow:
             )
             summary = self.correlator.get_summary(organized_timeline)
             
+            # Capture comprehensive data if screenshots were taken
+            comprehensive_data = {}
+            if self.artifacts.get("screenshots"):
+                # Get comprehensive data from last screenshot
+                last_screenshot = self.artifacts["screenshots"][-1]
+                if "comprehensive_data_path" in last_screenshot:
+                    import json
+                    try:
+                        with open(last_screenshot["comprehensive_data_path"], 'r') as f:
+                            comprehensive_data = json.load(f)
+                    except Exception as e:
+                        self.logger.warning(f"Could not load comprehensive data: {e}")
+            
             # Package results
             results = {
                 "success": success,
@@ -152,7 +165,8 @@ class CursorFlow:
                 "browser_events": self.timeline,  # Raw browser events
                 "server_logs": server_logs,       # Raw server logs
                 "summary": summary,               # Basic counts
-                "artifacts": self.artifacts
+                "artifacts": self.artifacts,
+                "comprehensive_data": comprehensive_data  # Complete page intelligence
             }
             
             self.logger.info(f"Test execution completed: {success}, timeline events: {len(organized_timeline)}")
@@ -669,6 +683,15 @@ class CursorFlow:
     
     async def _execute_actions(self, actions: List[Dict]) -> bool:
         """Execute list of declarative actions"""
+        # Validate all actions before execution
+        from .action_validator import ActionValidator, ActionValidationError
+        
+        try:
+            actions = ActionValidator.validate_list(actions)
+        except ActionValidationError as e:
+            self.logger.error(f"Action validation failed: {e}")
+            return False
+        
         try:
             for action in actions:
                 await self._execute_single_action(action)
@@ -678,44 +701,26 @@ class CursorFlow:
             return False
     
     async def _execute_single_action(self, action: Dict):
-        """Execute a single declarative action and track it"""
+        """
+        Execute a single declarative action and track it
+        
+        Pass-through architecture: CursorFlow handles a few special actions,
+        then passes everything else directly to Playwright Page API.
+        """
         action_start = time.time()
         
         try:
-            # Navigation actions
-            if "navigate" in action:
+            # Extract action type
+            action_type = action.get('type') or list(action.keys())[0]
+            
+            # CursorFlow-specific actions (not direct Playwright methods)
+            if action_type == "navigate":
                 url = action["navigate"]
                 if isinstance(url, dict):
                     url = url["url"]
                 await self.browser.navigate(url)
                 
-            # Interaction actions
-            elif "click" in action:
-                selector = action["click"]
-                if isinstance(selector, dict):
-                    selector = selector["selector"]
-                await self.browser.click(selector)
-                
-            elif "fill" in action:
-                config = action["fill"]
-                await self.browser.fill(config["selector"], config["value"])
-                
-            elif "type" in action:
-                config = action["type"]
-                await self.browser.type(config["selector"], config["text"])
-                
-            # Waiting actions
-            elif "wait" in action:
-                await asyncio.sleep(action["wait"])
-                
-            elif "wait_for" in action:
-                selector = action["wait_for"]
-                if isinstance(selector, dict):
-                    selector = selector["selector"]
-                await self.browser.wait_for_element(selector)
-                
-            # Capture actions
-            elif "screenshot" in action:
+            elif action_type == "screenshot":
                 screenshot_config = action["screenshot"]
                 
                 # Handle both string and dict formats
@@ -731,18 +736,23 @@ class CursorFlow:
                     name = "screenshot"
                     options = None
                 
+                # v2.1: No need for additional_selectors - we capture ALL visible elements now
                 screenshot_data = await self.browser.screenshot(name, options)
                 self.artifacts["screenshots"].append(screenshot_data)
                 
-            elif "authenticate" in action:
+            elif action_type == "authenticate":
                 if self.auth_handler:
                     await self.auth_handler.authenticate(self.browser.page, action["authenticate"])
+            
+            # Pass-through to Playwright Page API (hover, dblclick, press, etc.)
+            else:
+                await self._execute_playwright_action(action_type, action)
                     
             # Record action in timeline
             self.timeline.append({
                 "timestamp": action_start,
                 "type": "browser",
-                "event": list(action.keys())[0],
+                "event": action_type,
                 "data": action,
                 "duration": time.time() - action_start
             })
@@ -751,6 +761,47 @@ class CursorFlow:
             self.logger.error(f"Action failed: {action}, error: {e}")
             raise
     
+    async def _execute_playwright_action(self, action_type: str, action: Dict):
+        """
+        Pass-through to Playwright Page API
+        
+        Enables using any Playwright method without hardcoding them:
+        - hover, dblclick, drag_and_drop
+        - press, focus, blur, check, uncheck
+        - evaluate, route, expose_function
+        - And 90+ more methods
+        
+        See: https://playwright.dev/python/docs/api/class-page
+        """
+        # Get the action config
+        action_config = action.get(action_type)
+        
+        # Check if method exists on Playwright Page
+        if hasattr(self.browser.page, action_type):
+            method = getattr(self.browser.page, action_type)
+            
+            # Call with appropriate args based on config format
+            if isinstance(action_config, str):
+                # Simple format: {"hover": ".selector"}
+                await method(action_config)
+            elif isinstance(action_config, dict):
+                # Config format: {"hover": {"selector": ".item", "timeout": 5000}}
+                await method(**action_config)
+            elif action_config is None:
+                # No args: {"reload": null}
+                await method()
+            else:
+                # Numeric or other: {"wait": 2}
+                await method(action_config)
+        else:
+            # Method doesn't exist on Page - provide helpful error
+            available_methods = [m for m in dir(self.browser.page) if not m.startswith('_') and callable(getattr(self.browser.page, m))]
+            raise AttributeError(
+                f"Unknown Playwright action: '{action_type}'\n"
+                f"Available methods: {', '.join(sorted(available_methods[:20]))}...\n"
+                f"Full API: https://playwright.dev/python/docs/api/class-page"
+            )
+    
     async def _cleanup_session(self, session_options: Dict):
         """Clean up browser session"""
         try:
