@@ -170,7 +170,12 @@ class MockupComparator:
                 # Save comparison report
                 report_path = self.artifacts_base / "mockup_comparisons" / f"{comparison_name}.json"
                 with open(report_path, 'w') as f:
-                    json.dump(comparison_report, f, indent=2, default=str)
+                    # Custom JSON encoder to handle numpy booleans/integers
+                    def json_serializer(obj):
+                        if hasattr(obj, 'item'):  # numpy types
+                            return obj.item()
+                        return str(obj)
+                    json.dump(comparison_report, f, indent=2, default=json_serializer)
                 
                 self.logger.info(f"Mockup comparison completed: {comparison_name}")
                 return comparison_report
@@ -1194,11 +1199,11 @@ class MockupComparator:
             improvement = improved_similarity - baseline_similarity
             
             return {
-                "baseline_similarity": baseline_similarity,
-                "improved_similarity": improved_similarity,
-                "improvement": improvement,
-                "improvement_percentage": round(improvement, 2),
-                "is_improvement": improvement > 0
+                "baseline_similarity": float(baseline_similarity),
+                "improved_similarity": float(improved_similarity),
+                "improvement": float(improvement),
+                "improvement_percentage": round(float(improvement), 2),
+                "is_improvement": bool(improvement > 0)  # Convert to Python bool
             }
             
         except Exception as e:
@@ -1219,10 +1224,10 @@ class MockupComparator:
         avg_similarity = sum(similarities) / len(similarities) if similarities else 0
         
         return {
-            "average_similarity": round(avg_similarity, 2),
+            "average_similarity": round(float(avg_similarity), 2),
             "viewports_tested": len(comparison_results),
             "best_viewport_match": max(comparison_results, key=lambda x: x.get("visual_diff", {}).get("similarity_score", 0)),
-            "needs_improvement": avg_similarity < 80  # Threshold for "good enough"
+            "needs_improvement": bool(avg_similarity < 80)  # Convert numpy.bool_ to Python bool
         }
     
     def _generate_improvement_recommendations(self, comparison_results: List[Dict]) -> List[Dict]:
