@@ -363,7 +363,7 @@ class CursorFlowUpdater:
     
     async def _migrate_configuration(self) -> bool:
         """Migrate configuration to new format if needed"""
-        config_file = self.project_dir / "cursorflow-config.json"
+        config_file = self.project_dir / ".cursorflow" / "config.json"
         
         if not config_file.exists():
             return True
@@ -417,9 +417,10 @@ class CursorFlowUpdater:
         backup_path = backup_dir / backup_name
         
         # Backup configuration and rules
-        if (self.project_dir / "cursorflow-config.json").exists():
+        config_file = self.project_dir / ".cursorflow" / "config.json"
+        if config_file.exists():
             shutil.copy2(
-                self.project_dir / "cursorflow-config.json",
+                config_file,
                 backup_path.with_suffix('.config.json')
             )
         
